/*-*-linux-c-*-*/

#ifndef CONNECTION_H
#define CONNECTION_H

#include <stdio.h>
#include <gtk/gtk.h>

#include "global.h"

typedef enum {
	GS_UNUSED, GS_WAITINGCONNECT, GS_CONNECTED, GS_QUEUED, GS_COMPLETE, 
	GS_CANCELLED, GS_FAILED, GS_DISKERROR, GS_NEEDOVERWRITE, GS_WAITINGPUSH, 
	GS_WAITINGGIV, GS_WAITINGPUSHGET, GS_FILENOTFOUND, GS_BUSY, GS_TIMEOUT
} connection_state;

typedef enum {
	CT_NONE, CT_INCOMING, CT_OUTGOING, CT_GET, CT_SEND
} connection_type;

typedef enum {
	RC_INVALID, RC_200, RC_206, RC_400, RC_404, RC_503
} response;

struct newtella_connection {
	connection_state state;
	connection_type type;
	response response_code;

	guint8 id;
	
	guint32 total_sent, total_dropped_to_send;
	guint32 total_handled, total_received_dropped, total_bad_received;
	guint32 handled, sent;
	/* guint32 dropped_to_send, received_dropped, bad_received; */
	
	char bTemporary;
	guint32 speed;
       

	time_t last_time;
	guint32 s;
	guint32 ip;
	guint16 port;

	GList *packet_queue;
	int queue_size;

	char buffer[MAX_PACKET_LENGTH];
	int buffer_len;
	char *b;

	gint read_tag, write_tag;
	
	struct file_found *fi;

	/* for download handling */
	int transmitted;
	int header_transmitted;
	int download_ok;
	/* download file path */
	int d_file;
	/* size of the uploaded file */
	guint32 u_file_size;
};

/* general */
int send_packet(int s, const char *buf, int len);
char *recv_packet(int s, int len);
int bytes_avail(int s);

int con_recv_packet(struct newtella_connection *con, int len);
int con_send_packet(struct newtella_connection *con, const char *buf, int len);
int con_send_from_queue(struct newtella_connection *con);
int con_send_to_all(char *buf, int len);
int con_add(guint32 ip, guint16 port);
int con_dec(void);
int con_remove(struct newtella_connection *con);
int con_down_remove(struct newtella_connection *con);
int con_activate(void);
int con_reset_buffer(struct newtella_connection *con);
int con_fix_ids(void);
int con_down_fix_ids(void);

#endif
