/*-*-linux-c-*-*/

/*
 *  gnewtellium - Newtella for Unix
 *  Copyright (C) 2001 Elias Athanasopoulos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "newtella.h"
#include "file.h"
#include "gui.h"
#include "guid.h"

/* General functions for Upload Connections */

struct file_entry *upload_find_file(guint32 indx)
{
	/* finds a file in the database
	   using its index.
	*/
	struct file_entry *fe;

	GList *iter;

	iter = g_list_first(gl_file_db);
	while (iter) {
		fe = iter->data;
		if (fe->index == indx)
			return fe;

		iter = g_list_next(iter);
	} 

	/* if we reach uphere then something 
	   _really_ bad has happened. :-(
	*/
	return NULL;
}
int upload_write(gpointer gdata, gint s, GdkInputCondition cond)
{
        struct newtella_connection *u_con = (struct newtella_connection *)gdata;
        char data[1024];
	int bytes_readed;

        if (cond == GDK_INPUT_EXCEPTION) {
                u_con->state = GS_FAILED;
		gdk_input_remove(u_con->write_tag);
                return -1;
        }
       
	bytes_readed = read(u_con->d_file, &data, 1024);
	
	if (bytes_readed) {
		con_send_packet(u_con, data, bytes_readed);
		u_con->total_sent += bytes_readed - 1;
	} else {
		u_con->state = GS_COMPLETE;
		gdk_input_remove(u_con->write_tag);
        }

	/* gui update */
	gui_update_up_con(u_con);

        return 1;
}

int upload_add(guint32 ip, guint16 port, guint32 indx, guid *_guid)
{
	char *up_file;
	char http_header[1024];
	struct file_entry *fe = upload_find_file(indx);
	struct newtella_connection *u_con;

	if (!fe)
		return -1;

	up_file = newtella_malloc(strlen(fe->path)+strlen(fe->name)+2);
	up_file = strcpy(up_file, fe->path);
	up_file = strcat(up_file, fe->name);

	u_con = newtella_malloc(sizeof(struct newtella_connection));
	u_con->ip = ip;
        u_con->port = port;
        u_con->total_sent = 0;
	u_con->packet_queue = NULL;
        u_con->queue_size = 0;
        u_con->buffer_len = 0;
        u_con->state = GS_WAITINGCONNECT;     
        u_con->id = ++gl_options->current_up_con;
	u_con->u_file_size = fe->size;

        gl_up_con = g_list_append(gl_up_con, u_con);
        
        gui_add_up_con_to_list(u_con, fe->name); 

        u_con->s = newtella_connect(ip, port);
	
	g_snprintf(http_header, 1024, "GIV %d/%p/%s\r\n\r\n", indx, _guid, fe->name);
	con_send_packet(u_con, http_header, strlen(http_header));

	u_con->d_file = open(up_file, O_RDONLY);

	u_con->last_time = time((time_t *) NULL);
	u_con->write_tag = gdk_input_add(u_con->s, GDK_INPUT_WRITE | GDK_INPUT_EXCEPTION, 
                      (GdkInputFunction) upload_write, (gpointer)u_con);

	u_con->state = GS_CONNECTED;

	return 1;
}

int upload_start(struct newtella_connection *con, char *buf)
{
	struct newtella_connection *u_con; 
	char *up_file;
	char *http_header = newtella_malloc(1024);

	/* find the requested file */
	struct file_entry *fe = upload_find_file(strtoul((buf+9), NULL, 10));

	if (!fe)
		return -1;
	
	up_file = newtella_malloc(strlen(fe->path)+strlen(fe->name)+2);
	up_file = strcpy(up_file, fe->path);
	up_file = strcat(up_file, fe->name);

	/* add connection to upload connections */
	u_con = con;
	u_con->u_file_size = fe->size;
        u_con->id = ++gl_options->current_up_con;
	gl_up_con = g_list_append(gl_up_con, u_con);
	gui_add_up_con_to_list(u_con, fe->name); 
	
        /* remove ordinary connection */
	gl_con = g_list_remove(gl_con, con);
        gui_remove_con_from_list(con->id-1);
        gl_options->current_con--;
        gdk_input_remove(con->read_tag);
        gdk_input_remove(con->write_tag);

	g_snprintf(http_header, 1024, "HTTP 200 OK\r\nServer: gnewtellium\r\nContent-type:application/binary\r\nContent-length: %d\r\n\r\n", fe->size);
	con_send_packet(u_con, http_header, strlen(http_header));

	u_con->d_file = open(up_file, O_RDONLY);

	u_con->write_tag = gdk_input_add(u_con->s, GDK_INPUT_WRITE | GDK_INPUT_EXCEPTION, 
                      (GdkInputFunction) upload_write, (gpointer)u_con);


	return 1;
}


